{      WRITELOG.TPU    (c) 1995 CSC Ltd.  All right reserved.          }
{ ----============================================================---- }
{ Part: Main program.                                                  }
{                                                                      }
{$F+}
unit WriteLog;

{ ----------------- Interface part --------------- }

Interface
uses Dos;

{$I TYPES.INC} {     }

{$I CONST.INC} { ⠭  }

Var
      Modes                                      : ModeArrayType;
      IntValues                                  : Array [0..35] of LongInt;
      StrValues                                  : Array [0..35] of String;
      LogFile                                    : Text;
      ErrorString                                : ^ErrorStringArray;

Function  UpCaseStr      (S:String)                       : String;
Function  UpCaseStrSel   (S:String; Mode:Byte; 
                          RussianUpCase:Boolean)          : String;
Function  LeadZeroForm   (W:Longint;L:Byte)               : String;
Function  LeadZero       (W:Longint)                      : String;
Function  Str2Byte       (S:String; Old:LongInt)          : LongInt;
Function  Hex2Byte       (H:String)                       : LongInt;
Function  Byte2Str       (B:LongInt)                      : String;
Function  FormatString   (S:String)                       : String;

Procedure SetErrStrArray (var P; Num: Byte);
Procedure OpenLogFile    (N,S:String);
Procedure CloseLogFile   (S:String);
Procedure WriteToLogFile (S:String;K:Byte);

{ -------------- Implementation part ------------- }

Implementation

Var
      WeekMode            : Boolean;
      DivideChar          : Char;
      II,JI,KI            : Integer;

Function UpCaseStrSel(S:String; Mode:Byte;
                      RussianUpCase:Boolean):String;
var
    I   : Byte;
    C   : Char;
    CB  : Byte absolute C;
    S1  : String;
    SL  : Byte absolute S1;
  Begin
    S1:=S;
    For I:=1 to SL do
      Begin
        C:=S[I];
        Case Mode of
          1 : begin
                if (C in ['a'..'z'])
                  then CB:=CB-32
                  else if RussianUpCase
                         then Case C of
                                ''..''            : CB:=CB-32;
                                ''..''            : CB:=CB-80;
                                '','','','','' : CB:=CB-1;
                              end;
              end;
          2 : begin
                 if (C in ['A'..'Z'])
                   then CB:=CB+32
                   else if RussianUpCase
                          then Case C of
                                 ''..''            : CB:=CB+32;
                                 ''..''            : CB:=CB+80;
                                 '','','','','' : CB:=CB+1;
                               end;
               end;
          end;
        S1[I] := C;
      end;
    UpCaseStrSel:=S1;
  end;

Function UpCaseStr(S:String):String;
  Begin
    UpCaseStr:=UpCaseStrSel(S,1,False);
  end;

Function LeadZeroForm(W:Longint;L:Byte) : String;
var
  S : String;
  SL: Byte absolute S;
  I : Byte;
begin
  Str(W:0,S);
  if SL<L then
    for I:=1 to L-SL do S:='0'+S;
  LeadZeroForm:=S;
end;

Function LeadZero(W:Longint) : String;
begin
  LeadZero:=LeadZeroForm(W,Modes[0].LeadZeroDefault);
end;

Function Str2Byte(S:String; Old:LongInt):LongInt;
Var
    B           : LongInt;
    C           : Integer;
  Begin
    Val(S,B,C);
    if C=0 then Str2Byte:=B
           else Str2Byte:=Old;
  end;

Function Byte2Str(B:LongInt):String;
var
    S   : String;
  begin
    Str(B:Modes[0].LenByteDefault,S);
    Byte2Str:=S;
  end;

Procedure SetErrStrArray (var P; Num: Byte);
  Begin
    ErrorString:=@P;
    MaxErrorString:=Num;
  end;

Function Hex2Byte(H:String):LongInt;
Var
       HL       : Byte absolute H;
       Result   : LongInt;
       C        : Char;
       CB       : Byte absolute C;
       I,J,K    : Byte;
  begin
    Result:=0;
    For I:=1 to HL do
      begin
        C:=UpCase(H[I]);
        Case C of
          '0'..'9': K:=CB-48;
          'A'..'F': K:=CB-55;
          else K:=0;
        end;
        Result:=(Result shl 4)+K;
      end;
    Hex2Byte:=Result;
  end;

Function Byte2Hex(B:LongInt):String;
Var
       I,J,K    : Byte;
       S        : String;
       SL       : Byte absolute S;
  begin
    S:='';
    For I:=1 to 8 do
      begin
        J:=B and $0000000F;
        S:=HexTable[J]+S;
        B:=((B and $FFFFFFF0) shr 4);
      end;
    For I:=1 to 7 do
      if S[I]='0' then S:=Copy(S,2,10)
                  else I:=7;
    if SL<Modes[0].HexLenDefault then
      for I:=1 to Modes[0].HexLenDefault-SL do S:='0'+S;
    Byte2Hex:=S;
  end;

Function WeekDayFormat(D:Byte):String;
  Begin
    With Modes[0] do
      begin
        if WeekDayMode then WeekDayFormat:=WeekDayLongString[D]
                       else WeekDayFormat:=WeekDayShortString[D];
      end;
  end;

Function MonthFormat(M:Byte):String;
Var
    MonthS              : String;
  Begin
    With Modes[0] do
      begin
        WeekMode:=False;
        Case MonthMode of
          0 : Begin
                MonthS:=LeadZeroForm(M,2);
                DivideChar:=DateNumDivideChar;
              end;
          1 : MonthS:=MonthShortString[M];
          2 : MonthS:=MonthLongString[M];
          3 : begin
                MonthS:=MonthShortString[M];
                WeekMode:=True;
              end;
          end;
        MonthFormat:=MonthS;
      end;
  end;

Function DayFormat(D:Byte):String;
  Begin
    DayFormat:=LeadZeroForm(D,2);
  end;

Function YearFormat(Y:Word):String;
Var
    YearS       : String;
  begin
    YearS:=LeadZeroForm(Y,4);
    if Modes[0].YearMode then YearS:=Copy(YearS,3,2);
    YearFormat:=YearS;
  end;

Function FormatDate(Year,Month,Day,WeekDay:Word):String;
Var
    YearS,MonthS,DayS,WeekDayS,R : String;
    C                            : Char;
  begin
    With Modes[0] do
      begin
        if MonthMode=0 then DivideChar:=DateNumDivideChar
                       else DivideChar:=DateDivideChar;
        MonthS:=MonthFormat(Month);
        DayS  :=DayFormat  (Day);
        C:=DivideChar;
        WeekDayS:=WeekDayFormat(WeekDay);
        YearS:=YearFormat(Year);
        Case DateMode of
          0 : R:=DayS+C+MonthS+C+YearS;
          1 : R:=DayS+C+MonthS;
          end;
        if WeekMode then R:=WeekDayS+C+R;
        FormatDate:=R;
      end;
  end;

Function HourFormat(H:Word):String;
Var
    HourS : String;
  begin
    If Modes[0].TimeFormMode then HourS:=LeadZeroForm(H,2)
                             else HourS:=LeadZeroForm(H,0);
    HourFormat:=HourS;
  end;

Function MinFormat(M:Word):String;
Var
    MinS : String;
  begin
    If Modes[0].TimeFormMode then MinS:=LeadZeroForm(M,2)
                             else MinS:=LeadZeroForm(M,0);
    MinFormat:=MinS;
  end;

Function SecFormat(S:Word):String;
Var
    SecS : String;
  begin
    If Modes[0].TimeFormMode then SecS:=LeadZeroForm(S,2)
                             else SecS:=LeadZeroForm(S,0);
    SecFormat:=SecS;
  end;

Function FormatTime(Hour,Min,Sec:Word):String;
Var
    HourS,MinS,SecS,Sec100S,R    : String;
    FormMode                     : Boolean;
    C                            : Char;
  begin
    With Modes[0] do
      begin
        C:=TimeDivideChar;
        Hours:=HourFormat(Hour);
        MinS:=MinFormat(Min);
        SecS:=SecFormat(Sec);
        Case TimeMode of
          0 : R:=HourS+C+MinS+C+SecS;
          1 : R:=HourS+C+MinS;
          2 : R:=MinS+C+SecS;
        end;
        FormatTime:=R;
      end;
  end;

Function LoadIntValue(N,Flag:Byte):String;
Var   S : String;
      L : Byte;
  begin
    L:=Modes[0].LenByteDefault;
    Case Flag of
      0 :  Begin
             S:=Byte2Str(IntValues[N]);
             LoadIntValue:=S;
           end;
      1 :  Begin
             S:=LeadZero(IntValues[N]);
             LoadIntValue:=S; {Copy(S,1,L);}
           end;
      2 :  Begin
             S:=Byte2Hex(IntValues[N]);
             Case Modes[0].HexMode of
               1 : LoadIntValue:='$'+S;
               2 : LoadIntValue:=S+'h';
               3 : LoadIntValue:=S+'H'
               else LoadIntValue:=S;
               end;
           end
      else S:=Copy('                    ',1,L);
      end;
  end;

{$I FORM_STR.INC}

Procedure WriteToLogFile(S:String;K:Byte);
Var
    S1 : String;
  begin
    S1:=FormatString(S);
    Writeln(LogFile,ErrorString^[K],S1);
  end;

Procedure OpenLogFile   (N,S:String);
  Begin
    Assign(LogFile,N);
    {$I-}
    Append(LogFile);
    {$I+}
    if IOResult<>0 then ReWrite(LogFile);
    WriteToLogFile('',0);
    WriteToLogFile(S,0);
  end;

Procedure CloseLogFile(S:String);
  begin
    WriteToLogFile(S,0);
    Close(LogFile);
  end;

begin
  SetErrStrArray (ErrorStringDef,19);
  For II:=0 to 35 do
    begin
      IntValues[II]:=0;
      StrValues[II]:='';
    end;
  Modes:=ModeData;
end.
{$F-}